const CommandExecutor = require("../CommandExecutor");

class BDKServerCleanup
{
	constructor()
	{
	}


	/**
	 * @param {Object} objServer 
	 */
	static async run(objServer)
	{
		await BDKServerCleanup._runServerCleanupAuxiliaryFunctionsBatch();
		await BDKServerCleanup._runCleanupBatch(objServer);
	}
	

	/**
	 * @param {Object} objServer
	 * @returns {string}
	 */
	static async _runCleanupBatch(objServer)
	{
		let strCommand = `
			ILOREST="/sbin/ilorest"
			"HPRCU="/sbin/hprcu"
		`;
 
		strCommand += BDKServerCleanup._diskCleaningScript(objServer);
		strCommand += BDKServerCleanup._networkingCleanupScript(objServer);

		const strCommandOutput = await CommandExecutor.execCommandProcess(strCommand);

		return strCommandOutput;
	}


	/**
	 * @param {Object} objServer
	 * @returns {string}
	 */
	static _networkingCleanupScript(objServer)
	{
		let strNetworkingCleaningCommand = `
			intelToUpdate=\`/bin/bootutil64e | tail -n +10 | grep -v PXE | wc -l\`
			"if [ \"\$intelToUpdate\" -eq \"0\" ]; then
				echo "No Intel NICs needed update"
			else
				/bin/bootutil64e -quiet  -up=pxe -all -file=/opt/bdk/BootIMG.FLB
			fi
		`;

		const objNICDetails = objServer.NICDetails;

		for(let strMACAddr in objNICDetails)
		{
			const objNIC = objNICDetails[strMACAddr];
		}

		return strNetworkingCleaningCommand;
	}


	/**
	 * @param {Object} objServer
	 * @returns {string}
	 */
	static _diskCleaningScript(objServer)
	{
		let strDiskCleaningCommand = "";
		const arrDisks = objServer.disks;

		if(arrDisks && arrDisks.length !== 0)
		{
			for(let objDisk of arrDisks)
			{
				if(objDisk.server_disk_type === BDKServerCleanup.DISK_TYPE_NVME)
				{
					strDiskCleaningCommand += `
						nvme format --ses=2 ${objDisk.device_path}
						if [ \$? == \"0\" ]; then
							echo Drive wiping succesfully performed with nvme tool, no need for dd wiping
						else
							dd if=/dev/zero of=${objDisk.device_path} bs=10M conv=notrunc &
						fi
					`;
				}
				else
				{
					strDiskCleaningCommand += `
						sg_sanitize --quick --crypto --wait /dev/${objDisk["server_disk_label"]}
						if [ $? == \"0\" ]; then
							echo Drive wiping succesfully performed with sg_sanitize tool, no need for dd wiping
						else
							dd if=/dev/zero of=/dev/".$arrDisk["disk_label"]." bs=10M conv=notrunc &
						fi
					`;
				}
			}

			strDiskCleaningCommand += `
				while [ $(ps -eo pid,cmd | grep "dd if=/dev/zero" | grep -v grep | wc -l) -ne "0" ]; do
					echo Waiting for dd to finish...
					sleep 1m;
				done
				echo Finished all disks
			`;
		}

		return strDiskCleaningCommand;
	}


	/**
	 * Runs a batch code containing auxiliary functions which are called during cleaning.
	 */
	static async _runServerCleanupAuxiliaryFunctionsBatch()
	{
		const strCommand =  `
			function ilo_login() {
				logincount=1
				while true; do
					timeout --kill-after=13 10 \${ILOREST} login >/dev/null 2>&1
					if [ $? -eq 0 ]; then
						echo 'Login successfull'
						break
					fi
					echo \"Waiting for ILO to be available, try #\${logincount}...\"
					sleep 5
					logincount=\$((logincount+1))
					if [ \${logincount} -eq 40 ]; then
						echo \"Unable to login on iLO. Exiting\"
						exit
					fi
				done
			}
		`;

		await CommandExecutor.execCommandProcess(strCommand);
	}

	static get DISK_TYPE_NVME()
	{
		return "NVME";
	}
}

module.exports = BDKServerCleanup;
